//
//  AppDelegate.swift
//  HelloWorld
//
//  Created by Phidgets on 2018-02-16.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import Cocoa
import Phidget22Swift

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    @IBOutlet var textView: NSTextView!
    let ch = Manager()
    
    //Error Window
    @IBOutlet weak var errorEventWindow: NSPanel!
    @IBOutlet var errorEventLog: NSTextView!
    @IBOutlet weak var errorEventCount: NSTextField!
    var errorCounter = 0
    
    //MARK: View events
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        do {
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
            
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func applicationShouldTerminateAfterLastWindowClosed(_ sender: NSApplication) -> Bool {
        return true
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        //Safely close example
        do{
            print("Closing")
            ch.attach.removeAllHandlers()
            ch.detach.removeAllHandlers()
            try ch.close()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event handlers
    func attach_handler(sender: Manager, channel: Phidget){
        do{
            let deviceName = try channel.getDeviceName()
            let deviceSerialNumber = try channel.getDeviceSerialNumber()
            let deviceChannel = try channel.getChannel()
            
            DispatchQueue.main.async {
                self.textView.textStorage?.beginEditing()
                let outputString = NSAttributedString(string: String(format:"Hello %@, Serial Number: %d, Channel: %d\n", deviceName, deviceSerialNumber, deviceChannel))
                self.textView.textStorage?.append(outputString)
                self.textView.textStorage?.endEditing()
                self.textView.scrollToEndOfDocument(self)
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Manager, channel: Phidget){
        do{
            let deviceName = try channel.getDeviceName()
            let deviceSerialNumber = try channel.getDeviceSerialNumber()
            let deviceChannel = try channel.getChannel()
            
            DispatchQueue.main.async {
                //Adjusting window height
                var frame = self.window.frame
                frame.size.height = self.window.minSize.height
                self.window.setFrame(frame, display: true, animate: false)
                self.window.center()
                
                self.textView.textStorage?.beginEditing()
                let outputString = NSAttributedString(string: String(format:"Goodbye %@, Serial Number: %d, Channel: %d\n", deviceName, deviceSerialNumber, deviceChannel))
                self.textView.textStorage?.append(outputString)
                self.textView.textStorage?.endEditing()
                self.textView.scrollToEndOfDocument(self)
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    
    //GUI Controls
    @IBAction func clear(_ sender: Any) {
        textView.textStorage?.setAttributedString(NSAttributedString(string: ""))
    }
    
    //MARK: Error
    func outputError(errorDescription:String, errorCode:UInt32){
        DispatchQueue.main.async {
            self.errorCounter += 1
            let outputString = NSAttributedString(string: String(format:"Error %d: %@\n", errorCode, errorDescription))
            self.errorEventLog.textStorage?.beginEditing()
            self.errorEventLog.textStorage?.append(outputString)
            self.errorEventLog.textStorage?.endEditing()
            self.errorEventCount.stringValue = "\(self.errorCounter)"
            if(!self.errorEventWindow.isVisible){
                self.errorEventWindow.setIsVisible(true)
                self.errorEventWindow.center()
            }
        }
    }
    
    @IBAction func clearError(_ sender: Any) {
        errorEventLog.textStorage?.setAttributedString(NSAttributedString(string: ""))
        errorCounter = 0
        errorEventCount.stringValue = "0"
    }
}

